﻿using System;
using System.IO;
using System.Collections.Generic;
using System.Text;

namespace EncodingConverter
{
    /// <summary>
    /// 工具类
    /// </summary>
    public class Tool
    {
        #region 获取文件编码
        /// <summary>
        /// 取得一个文本文件的编码方式。如果无法在文件头部找到有效的前导符，Encoding.Default将被返回。
        /// </summary>
        /// <param name="fileName">文件名</param>
        /// 
        public static Encoding GetEncoding(string fileName)
        {
            return GetEncoding(fileName, System.Text.Encoding.GetEncoding("gb2312"));
        }

        /// <summary>
        /// 取得一个文本文件流的编码方式。
        /// </summary>
        /// <param name="stream">文本文件流</param>
        /// <returns></returns>
        /// 
        public static Encoding GetEncoding(FileStream stream)
        {
            return GetEncoding(stream, Encoding.Default);
        }

        /// <summary>
        /// 取得一个文本文件的编码方式。
        /// </summary>
        /// <param name="fileName">文件名。</param>
        /// <param name="defaultEncoding">默认编码方式。当该方法无法从文件的头部取得有效的前导符时，将返回该编码方式。</param>
        /// <returns></returns>
        /// 
        public static Encoding GetEncoding(string fileName, Encoding defaultEncoding)
        {
            FileStream fs = new FileStream(fileName, FileMode.Open);
            Encoding targetEncoding = GetEncoding(fs, defaultEncoding);
            fs.Close();
            return targetEncoding;
        }

        /// <summary>
        /// 取得一个文本文件流的编码方式。

        /// </summary>
        /// <param name="stream">文本文件流。</param>

        /// <param name="defaultEncoding">默认编码方式。当该方法无法从文件的头部取得有效的前导符时，将返回该编码方式。</param>
        /// <returns></returns>
        /// 
        public static Encoding GetEncoding(FileStream stream, Encoding defaultEncoding)
        {
            Encoding targetEncoding = defaultEncoding;
            if (stream != null && stream.Length >= 2)
            {
                //保存文件流的前4个字节
                byte byte1 = 0;
                byte byte2 = 0;
                byte byte3 = 0;
                byte byte4 = 0;

                //保存当前Seek位置
                long origPos = stream.Seek(0, SeekOrigin.Begin);
                stream.Seek(0, SeekOrigin.Begin);
                int nByte = stream.ReadByte();
                byte1 = Convert.ToByte(nByte);
                byte2 = Convert.ToByte(stream.ReadByte());

                if (stream.Length >= 3)
                {
                    byte3 = Convert.ToByte(stream.ReadByte());
                }

                if (stream.Length >= 4)
                {
                    byte4 = Convert.ToByte(stream.ReadByte());
                }
                //根据文件流的前4个字节判断Encoding
                //Unicode {0xFF, 0xFE};
                //BE-Unicode {0xFE, 0xFF};
                //UTF8 = {0xEF, 0xBB, 0xBF};

                if (byte1 == 0xFE && byte2 == 0xFF)//UnicodeBe
                {
                    targetEncoding = Encoding.BigEndianUnicode;
                }

                if (byte1 == 0xFF && byte2 == 0xFE && byte3 != 0xFF)//Unicode
                {
                    targetEncoding = Encoding.Unicode;
                }

                if (byte1 == 0xEF && byte2 == 0xBB && byte3 == 0xBF)//UTF8
                {
                    targetEncoding = Encoding.UTF8;
                }

                //恢复Seek位置
                stream.Seek(origPos, SeekOrigin.Begin);
            }
            return targetEncoding;
        }
        #endregion

        #region 文件编码转换
        /// <summary>
        /// 转换文件编码
        /// </summary>
        /// <param name="strPath">文件夹路径</param>
        /// <param name="strSuffix">后缀名</param>
        /// <param name="fromEncoding">开始编码</param>
        /// <param name="toEncoding">转换后编码</param>
        public static void ConvertFile(string strPath, string strSuffix, Encoding fromEncoding, Encoding toEncoding)
        {
            StreamReader sr = null;
            StreamWriter sw = null;
            string[] strFileSuffix = null;
            List<string> FileSuffix = new List<string>();
            List<string> strFileList = new List<string>();
            try
            {
                strFileSuffix = new string[strSuffix.Replace("*", "").Split(';').Length];
                strFileSuffix = strSuffix.Replace("*", "").Split(';');
                for (int i = 0; i < strFileSuffix.Length; i++)
                {
                    FileSuffix.Add(strFileSuffix[i]);
                }
                GetFileList(strPath, FileSuffix, ref strFileList);
                foreach (string file in strFileList)
                {
                    if (GetEncoding(file) == fromEncoding)
                    {
                        ConvertFile(file, fromEncoding, toEncoding, ref sr, ref sw);
                    }
                }
            }
            catch (Exception e)
            {
                throw e;
            }
        }

        /// <summary>
        /// 转换文件编码
        /// </summary>
        /// <param name="strFile">文件路径</param>
        /// <param name="fromEncoding">开始编码</param>
        /// <param name="toEncoding">转换后编码</param>
        public static void ConvertFile(string strFile, Encoding fromEncoding, Encoding toEncoding, ref StreamReader sr, ref StreamWriter sw)
        {
            sr = new StreamReader(strFile, System.Text.Encoding.GetEncoding("GB2312"));
            string source = sr.ReadToEnd();
            sr.Close();
            byte[] byteArray = fromEncoding.GetBytes(source);
            byte[] result = Encoding.Convert(fromEncoding, toEncoding, byteArray);
            string str = toEncoding.GetString(result);
            sw = new StreamWriter(strFile, false, toEncoding);
            sw.Write(str);
            sw.Close();
        }
       
        /// <summary>
        /// 获取指定目录路径(包括子目录)下所有指定后缀名的文件路径
        /// </summary>
        /// <param name="strPath">目录路径</param>
        /// <param name="strSuffix">后缀名</param>
        /// <returns></returns>
        public static void GetFileList(string strPath, List<string> FileSuffix, ref List<string> strFileList)
        {
            try
            {
                // 得到源目录的文件列表，该里面是包含文件以及目录路径的一个数组
                string[] fileList = System.IO.Directory.GetFileSystemEntries(strPath);
                // 遍历所有的文件和目录
                foreach (string file in fileList)
                {
                    // 先当作目录处理如果存在这个目录就重新调用GetFileList
                    if (System.IO.Directory.Exists(file))
                    {
                        GetFileList(file, FileSuffix, ref strFileList);
                    }
                    else
                    {
                        if (CheckFileSuffix(file,FileSuffix))
                        {
                            strFileList.Add(file);
                        }
                    }
                }
            }
            catch (Exception e)
            {
                throw e;
            }
        }

        /// <summary>
        /// 检查文件后缀名
        /// </summary>
        /// <param name="File">文件名</param>
        /// <param name="FileSuffix">后缀名</param>
        /// <returns></returns>
        public static bool CheckFileSuffix(string File, List<string> FileSuffix)
        {
            string temp = File.Substring(File.LastIndexOf('.'), (File.Length - File.LastIndexOf('.')));
            return FileSuffix.Contains(temp);
        }
        #endregion
    }
}
